#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
*****************************************************************************************************
LIRIS Evaluator of human activity recognition and localisation algorithms - harl
Command line application (use : harlcore.py)

Author : Tristan Delizy,
*****************************************************************************************************
"""
"""
*****************************************************************************************************
this script is the command line based interface for the module harlcore.py

*****************************************************************************************************
"""

#====================================================================================================#
#IMPORTS
#====================================================================================================#
import sys, os, glob, re, getopt
import xml.etree.ElementTree as xml
import harlcore as hc

#====================================================================================================#
#GLOBAL CONSTANTS
#====================================================================================================#
DEFAULT_PERF_VALUE_THRESHOLD = 0.1
DEFAULT_PLOT_STEP = 0.05

VERSION = "v1.0"

#----------------------------------------------------------------------------------------------------usage
#print to stout info and man about this application
def usage():
  if sys.platform == "linux" or sys.platform == "linux2":
    bold = "\033[1m"
    reset = "\033[0;0m"
  else:
    bold = ""
    reset = ""
  print "\n\n\t\t\t"+bold+"harl.py - help\n"+reset

  print bold+"\nNAME"+reset
  print "\tLIRIS-HARL file evaluator tool "+VERSION

  print bold+"\nSYNOPSIS"+reset
  print "\t"+bold+"harl [ mode ] [ --options ]"+reset

  print bold+"\nDESCTIPTION"+reset
  print "\tEvaluation application for the ICPR 2012 competition : this scientific competition is centred on the recognition of complex human activities in a realistic ",\
  "surveillance setting and in an office environnement. The participant of the competition used a set of video, were each video may contain one or several actions.",\
  "the purpose of this application is to analyse the xml result files of the participants. The mesures used are precision and recall to considere correctly information",
  print "loss and wrong detection. The final rank of a detection data set will be set with the F-Score (or harmonic means) method."

  print bold+"\nMODE"+reset
  print "\t"+bold+"[ pervalue | plot | basic | matrix] "+reset+": mode of processing : perfvalue process the score of input dataset with fixed thresholds,",\
  "plot generates text files in .\\output with values for thresholds variation. Basic mode scores input dataset without spatial or temporal localisation. ",\
  "The matrix mode print on standard output the confusion matrix for the detection dataset row-normalized."

  print bold+"\nOPTIONS"+reset
  print bold+"\tthresholds : --t srt_spt_trt_tpt "+reset+"\nperfvalue : each threshold is replaced with its value. ",\
  "\nplot : each threshold is formatted this way : \"begin:end:step\" with step optionnal.",\
  "\nbasic : option not supported.",\
  "\nmatrix : option not supported."

  print bold+"\tFixthresholds : --fix trt_tpt_srt_spt"+reset+"\nperfvalue : option not supported.",\
  "\nplot : each threshold is replaced with the value it will keep during other thresholds variation (default : 0.1_0.1_0.1_0.1).",\
  "\nbasic : option not supported.",\
  "\nmatrix : option not supported."

  print bold+"\tOut directory : --outdir name"+reset+"\nperfvalue : option not supported.",\
  "\nplot : name of the sub-directory of current where the results will be saved (default is \"outdir\").",\
  "\nbasic : option ot supported.",\
  "\nmatrix : option not supported."

  print bold+"\tInput files : --files inputFiles @ groundTruthFiles"+reset+"\nfor all modes",\
  "list of files/directories separated by spaces (direcories will be develloped searching for xml files) input data set and ground truth dataset are separated by the \"@\" symbol."

  print bold+"\tHelp : -h"+reset+"\n any -h will instantly bring up this help."

  print bold+"\nOUTPUT"+reset
  print "\tIn perfvalue and basic modes, the only output (not on stderr) is a line with the 3 computed values in that order : "+bold+"recall precision fscore."+reset,\
  "\nIn plot mode the outputs are text files in the output directory and 8 values : \n"+bold+"recall precision fscore "+reset+"(these 3 are perfvalue at fix values given)",\
  "\n"+bold+"trt tpt srt spt"+reset+" (average of fscore for threshold variation) \n"+bold+"combined_performance"+reset+" (average of trt tpt srt spt) ."

  print bold+"\nEXAMPLES"+reset
  print "\tif you have downloaded the file.py, think about chmod +x file.py, unless that, you will need to prefix the command by \"python \"."
  print "\nto use the plot mode on file G.xml and D.xml whith the thresholds variation from 0.25 to 0.75 with a step of 0.05 and fix thresholds to 0.3 for variation: "
  print "./harl plot --t 0.25:0.75:0.05_0.25:0.75:0.05_0.25:0.75:0.05_0.25:0.75:0.05 --fix 0.3_0.3_0.3_0.3 --files D.xml @ G.xml"

#----------------------------------------------------------------------------------------------------incorrectCall
#exit when launch with bad params.
def incorrectCall(s):
  if s != None:
    print >> sys.stderr, "ERROR : "+s+"\nplease use option -h to get some help."
  else:
    print >> sys.stderr, "ERROR, please use option -h to get some help."
  sys.exit(2)

#----------------------------------------------------------------------------------------------------devellopFilePath
#take a set of xml files and directories and replace all directories by xml files inside
def devellopFilePath(rep):
  filenames=sorted(glob.glob(rep+"*.xml"))
  return filenames

#----------------------------------------------------------------------------------------------------main()
#main function of the programm : working in command line (see usage()) :
def main():
  mode = ''
  outdir = ""
  concat = ""
  filenames = ""
  Dfiles = set()
  Gfiles = set()
  fix = None
  trt = DEFAULT_PERF_VALUE_THRESHOLD
  tpt = DEFAULT_PERF_VALUE_THRESHOLD
  srt = DEFAULT_PERF_VALUE_THRESHOLD
  spt = DEFAULT_PERF_VALUE_THRESHOLD


  print >> sys.stderr,"LIRIS-HARL file evaluator tool "+VERSION

  #if no args is passed will bring up the help
  if len(sys.argv) < 2:
    usage()
    sys.exit(1)

  for i in range(len(sys.argv)):
    #any '-h' as a command argument will instantly bring help
    if sys.argv[i] == "-h" :
      usage()
      sys.exit(1)
    concat+=sys.argv[i]

  #mode detection
  if sys.argv[1] == "perfvalue":
    mode = "perfvalue"
  elif sys.argv[1] == "plot":
    mode = "plot"
  elif sys.argv[1] == "basic":
    mode = "basic"
  elif sys.argv[1] == "matrix":
    mode = "matrix"
  else:
    incorrectCall("cannot find execution mode.")
  print >> sys.stderr, "mode detected :", mode

  try:
    #get args
    opts, args = getopt.getopt(sys.argv[2:], "h", ["help", "outdir=", "t=", "fix=", "files"])
  except getopt.GetoptError, err:
    # print help information and exit:
    print >> sys.stderr, str(err) # will print something like "option -a not recognized"
    incorrectCall(None)

  #command interpretation
  for o, a in opts:
    if o in ("-h", "--help"):
      usage()
      sys.exit(1)
    elif o == "--outdir":
      if mode == "plot":
        result = re.match('([0-9]|[a-z]|[A-Z]|_|\-|=|@|^|)|(|\]|\[|\,|\+|\.|\:|\;|\?|\!|\<|\>)+',a)
        if result != None:
            outdir = a
        else:
            incorrectCall("outir contain illegal symbols")
      else:
        incorrectCall("outdir option not used in "+mode+" mode.")
    elif o == "--t":
      if mode == "perfvalue":
        try:
         print >> sys.stderr, "\nthreshold recognition :"
         print >> sys.stderr, "a = \"",a,"\""
         thresholds = a.split('_')
         print >> sys.stderr, "split =", a.split("_")
         trt=float(thresholds[0].replace(',','.'))
         tpt=float(thresholds[1].replace(',','.'))
         srt=float(thresholds[2].replace(',','.'))
         spt=float(thresholds[3].replace(',','.'))
        except:
	       incorrectCall("unable to read thresholds (--t arg)")
        if trt <= 1 and trt >=0:
	       print >> sys.stderr, "trt = ", trt
        else:
	       print >> sys.stderr, "trt invalid : default value used"
	       trt = DEFAULT_PERF_VALUE_THRESHOLD
        if tpt <= 1 and tpt >=0:
	       print >> sys.stderr, "tpt = ", tpt
        else:
	       print >> sys.stderr, "tpt invalid : default value used"
	       tpt = DEFAULT_PERF_VALUE_THRESHOLD
        if srt <= 1 and srt >=0:
	       print >> sys.stderr, "srt = ", srt
        else:
	       print >> sys.stderr, "srt invalid : default value used"
	       srt = DEFAULT_PERF_VALUE_THRESHOLD
        if spt <= 1 and spt >=0:
	       print >> sys.stderr, "spt = ", spt
        else:
	       print >> sys.stderr, "spt invalid : default value used"
	       spt = DEFAULT_PERF_VALUE_THRESHOLD
      elif mode == "plot":
        result = [0 for i in range(4)]
        t = [0 for i in range(4)]
        for i in range(4):
            result[i] = re.match('(0((\.|\,)[0-9]+)?)((:0(\.|\,)[0-9]+)|:1)(:0(\,|\.)[0-9]+)?', a.split('_')[i])
            if result[i] != None:
    	       begin = float(result[i].group(1).replace(',','.'))
    	       end = float(result[i].group(4).split(':')[1].replace(',','.'))
    	       step = DEFAULT_PLOT_STEP
    	       if result[i].group(7) != None:
                    step = float(result[i].group(7).split(':')[1].replace(',','.'))
    	       t[i] = [begin,end,step]
    	       #the step is optionnal so this check cannot be done regex (easly)
    	       if begin >1 or end>1 or step>1:
                    incorrectCall("value for thresholds must be in [0..1]")
            else:
    	           incorrectCall("cannot read tr")
	trt = t[0]
	tpt = t[1]
	srt = t[2]
	spt = t[3]
      else:
        incorrectCall("mode error")
    elif o == "--fix":
      if mode == "plot":
	   fix = a
      else:
	   incorrectCall("fix option not used only in plot mode.")
    elif o == "--files":
      #nothing is done here, but we need to process --file for getopts
      nothing = None
    else:
      incorrectCall("unhandled option ("+o+")")

  #get the xml files
  result=re.match('.*(--files).+',concat)
  if result != None:
    t = 0
    while t<len(sys.argv) and sys.argv[t] != "--files":
      t += 1
    filenames = ""
    t+=1
    while t<len(sys.argv):
      filenames += sys.argv[t]+' '
      t+=1
    filenames = filenames.split(' ')

    separator = 0
    for i in range(len(filenames)):
      if filenames[i] == '@':
	separator = 1
      elif filenames[i] != '':
	if separator == 0:
	  if re.match('.*\.xml', filenames[i]):
	    Dfiles.add(filenames[i])
	  else:
	    files=devellopFilePath(filenames[i])
	    for i in range(len(files)):
	      Dfiles.add(files[i])
	else:
	  if re.match('.*\.xml', filenames[i]):
	    Gfiles.add(filenames[i])
	  else:
	    files=devellopFilePath(filenames[i])
	    for i in range(len(files)):
	      Gfiles.add(files[i])
    if len(Gfiles)==0:
      incorrectCall("no ground truth files detected")
    elif len(Dfiles)==0:
      incorrectCall("no detection files detected")
    else:
      print >> sys.stderr, "detection data files : "
      for cle in Dfiles:
	print >> sys.stderr, "\t"+cle
      print >> sys.stderr, "ground truth data files : "
      for cle in Gfiles:
	print >> sys.stderr, "\t"+cle
  else:
    incorrectCall("missing \"--files files\"")

  #fix the fix string
  if fix == None:
    fix = "0.1_0.1_0.1_0.1"

  if mode == "plot" :
    if trt == DEFAULT_PERF_VALUE_THRESHOLD :
      trt = [0,1,0.1]
    if tpt == DEFAULT_PERF_VALUE_THRESHOLD :
      tpt = [0,1,0.1]
    if srt == DEFAULT_PERF_VALUE_THRESHOLD :
      srt = [0,1,0.1]
    if spt == DEFAULT_PERF_VALUE_THRESHOLD :
      spt = [0,1,0.1]

  #launch the computing
  try:
    if mode == "perfvalue":
        HarlEvaluator = hc.Evaluator(mode, Dfiles, Gfiles, trt, tpt, srt, spt)
        HarlEvaluator.Process()
    elif mode == "plot":
        HarlEvaluator = hc.Evaluator(mode, Dfiles, Gfiles, trt, tpt, srt, spt, outdir, fix)
        HarlEvaluator.Process()
    elif mode == "basic":
        HarlEvaluator = hc.Evaluator(mode, Dfiles, Gfiles)
        HarlEvaluator.Process()
    elif mode == "matrix":
        HarlEvaluator = hc.Evaluator("gui")
        mat = HarlEvaluator.Confusion_Process(Dfiles, Gfiles)
        if mat != None :
            lines_sum = []
            for col in range(10):
                s = 0
                for line in range(10):
                    s += mat[line][col]
                lines_sum.append(s)

        print "\n\n\n"
        for i in range(10):
            for j in range(10):
                if lines_sum[i] != 0:
                    print str(int(round(float(mat[j][i])/lines_sum[i]*100))).rjust(4),
                else:
                    print str(0).rjust(4),
            print "\n"

  except hc.harlException as e :
    print >> sys.stderr, e.cause+"\nsystem exiting..."


if __name__ == '__main__':
  main()

